def easy_mikro_next(base_url: str, minio_url: str):
    """Build mikro_next when the parameters are known."""


def build_local_mikro_next(base_url, storage_path: str):
    """Use mikro locally, with local storage.

    Attention, the storage path here needs to map to a local
    storage that is also accessible from the docker container.


    """
    raise NotImplementedError("Not implemented yet")
